// 
// JetHott pour telemesure Graupner Hott
// 
// Gestion messagerie Hott
//
// Auteur : www.thyzoon.fr 
//
// 2011-2019  
//

#include <EEPROM.h>  
#include "Message.h"
#include "Tension_Lipo.h"

#define VERSION         "0.24"

#define LEDPIN_OFF              PORTB &= ~(1<<5);
#define LEDPIN_ON               PORTB |= (1<<5);

// -------------------------------------
// Valeurs par défaut stockées en EEPROM
//
#define TENSION_MIN     		  750   // 720 = 7.20V pour 2 elements Lipo
#define REPETITION_ALARME   	30    // 30 s
#define ALARME_BAT_2        	1     // 0 = OFF / 1 = ON
#define ALARME_BAT_3        	1     // 0 = OFF / 1 = ON
#define IMPULS_LITRE          11800 // Nombre d'impulsions par litre du debitmetre (Valeur typique du BioTech FCH-M-PP
#define CAPACITE_RESERVOIR		2000  // Capacite du reservoir en mililitre (ml)
#define SEUIL_RESERVOIR_CARB  500   // Seuil d'alarme du réservoir
#define ALARME_CARBURANT_BAS  1     // 0 = OFF / 1 = ON
#define MODE_CARBURANT        0     // 0 = Reservoir / 1 = Quantite
#define QUANTITE_CARBURANT    2000  // Quantite de carburant a consommer
// Mode Reservoir : 
//   Parametres : Capacite du reservoir 
//                Seuil mini pour declenchement de l'alarme
// Mode Quantite : 
//   Parametres : Quantite à consommer , puis declenchement de l'alarme  
              

// Adresses des parametres en EEPROM, tous les parametres sont stockes sur 16 bits.  
#define adr_eprom_init                 0     // ad pour initialisation EEPROM a la 1iere mise sous tension 
#define adr_eprom_seuil_tension_alarme 2     // ad Tension seuil alarme
#define adr_eprom_repetition           4     // ad Repetition
#define adr_eprom_alarm_bat2		       6     // ad On/Off alarme accu 2
#define adr_eprom_alarm_bat3           8     // ad On/Off alarme accu 3
#define adr_eprom_impulsions_litre    10     // ad impulsions par litre du debitmetre
#define adr_eprom_capa_reservoir      12     // ad capacite du reservoir
#define adr_eprom_seuil_reservoir     14     // ad Seuil d'alarme du réservoir
#define adr_eprom_alarm_carburant     16     // ad On/Off alarme seuil carburant
#define adr_eprom_mode_carburant      18     // ad Mode carburant
#define adr_eprom_qt_carburant        20     // ad Quantite de carburant a consommer

// buffer message
static uint8_t _hott_serial_buffer[173];   //création d'une variable tampon pour stocker les struct

// pointer des structures vers le tampon "_hott_serial_buffer"
struct HOTT_GAM_MSG             *hott_gam_msg = (struct HOTT_GAM_MSG *)&_hott_serial_buffer[0];
struct HOTT_TEXTMODE_MSG	*hott_txt_msg =	(struct HOTT_TEXTMODE_MSG *)&_hott_serial_buffer[0];

// Parametres memorises en EEPROM (relues a la mise sous tension)
word seuil_tension_alarme;       // seuil tension alarme 
word temps_repetition;           // Repetition alarme
word alarme_lipo_2;              // On/Off alarme pour LiPo 2
word alarme_lipo_3;              // On/Off alarme pour LiPo 3
word impulsions_litre;           // nb impulsion / litre du debitmetre (valeur d'origine ou corrigee)
word capa_reservoir;             // Capacite du reservoir
word seuil_reservoir;            // Seuil alarme carburant
word alarme_carburant;           // On/Off alarme carburant
word mode_gst_carburant;         // Mode de gestion du carburant
word qt_carburant;               // Quantite de carburant a consommer

extern word volume_carburant;    // Volume actuel de carburant dans le reservoir
extern float imp_ml;             // nb d'impulsion par ml
extern float conso_carburant;    // carburant consomme en ml 

// For communication
static uint8_t octet1 = 0;  // reception
static uint8_t octet2 = 0;  // reception

// Variables
extern float lipo1; // Tension sur chacun des éléments
extern float lipo2;
extern float lipo3;
extern byte bip_alarme;
uint8_t Jauge = 0;  // Jauge carburant


extern TensionLipo tension;

GMessage::GMessage(){

}

void GMessage::init(){
  
// Initialisation COM 19200 bauds pour recepteur HOTT GRxx
  Serial.begin (SERIAL_COM_SPEED); 

// Sauvegardedes parametres en EEPROM 
/*
 !WARNING!
 Writing takes 3.3ms.
 Maximum life of the EEPROM is 100000 writings
 */
  // ******************************************************
  // Test for 1st time init of the Arduino (First power on)
  int test = read_eprom(adr_eprom_init);
  if (test != 67)
  {
    write_eprom(adr_eprom_init,67);
    write_eprom(adr_eprom_seuil_tension_alarme,TENSION_MIN);  // Seuil par défaut	
	  write_eprom(adr_eprom_repetition,REPETITION_ALARME);      // Temps repetition alarme
	  write_eprom(adr_eprom_alarm_bat2,ALARME_BAT_2);           // Alarme bat 2 On / OFF 
    write_eprom(adr_eprom_alarm_bat3,ALARME_BAT_3);           // Alarme bat 3 On / OFF 
    write_eprom(adr_eprom_impulsions_litre,IMPULS_LITRE);     // Impulsion / litre
    write_eprom(adr_eprom_capa_reservoir,CAPACITE_RESERVOIR);     // Capacite du reservoir en mililitre (ml)
    write_eprom(adr_eprom_seuil_reservoir,SEUIL_RESERVOIR_CARB);  // Seuil d'alarme du réservoir
	  write_eprom(adr_eprom_alarm_carburant,ALARME_CARBURANT_BAS);  // Alarme carburant On / OFF 
	  write_eprom(adr_eprom_mode_carburant,MODE_CARBURANT);    // Mode Reservoir
	  write_eprom(adr_eprom_qt_carburant,QUANTITE_CARBURANT);  // Quantite de carburant a consommer
  }
  
  // Lecture des valeurs en EEPROM
  seuil_tension_alarme = read_eprom(adr_eprom_seuil_tension_alarme);    //  seuil accu
  temps_repetition = read_eprom(adr_eprom_repetition);        // temps de repetition entre alarme
  alarme_lipo_2 = read_eprom(adr_eprom_alarm_bat2);           // Alarme bat 2 On / OFF  
  alarme_lipo_3 = read_eprom(adr_eprom_alarm_bat3);           // Alarme bat 3 On / OFF
  impulsions_litre = read_eprom(adr_eprom_impulsions_litre);  // impulsion / litre
  capa_reservoir = read_eprom(adr_eprom_capa_reservoir);      // Capacite du reservoir en mililitre (ml)
  seuil_reservoir = read_eprom(adr_eprom_seuil_reservoir);    // Seuil d'alarme du réservoir
  alarme_carburant = read_eprom(adr_eprom_alarm_carburant);   // Alarme carburant On / OFF
  mode_gst_carburant = read_eprom(adr_eprom_mode_carburant);  // 0 = Reservoir / 1 = Quantite 
  qt_carburant = read_eprom(adr_eprom_qt_carburant);          // Quantite de carburant a consommer
  
  // Init. pour certaines données de COM Hott 
  setLipo (0,1);
  setLipo (0,2);
  setLipo (0,3);      
  setLipo (0,4);  
  setLipo (0,5);  
  setLipo (0,6);  
  setAltitudeInMeters (0);
  setClimbrate (0);
  setClimbrate_M3 (0);
  setTemp (0,2);
  setVoltage1(0); 
  setVoltage2(0);
  setMainVoltage(0);
  setFuelPercent (100); 
  setFuelMilliliters(capa_reservoir);
}

// Lecture d'un mot de 16 bits en EEPROM
uint16_t GMessage::read_eprom(int address){
  return  (uint16_t) EEPROM.read(address) * 256 + EEPROM.read(address+1) ;
}

// Ecriture d'un mot de 16 bits en EEPROM
void GMessage::write_eprom(int address,uint16_t val){
  EEPROM.write(address, val  / 256);
  EEPROM.write(address+1,val % 256 );
}

// Initialisation message GAM (General Air Module)
void GMessage::init_gam_msg(){
  //met tous à Zero, puis on modifie les constantes
  memset(hott_gam_msg, 0, sizeof(struct HOTT_GAM_MSG));   
  hott_gam_msg->start_byte = 0x7c;
  hott_gam_msg->gam_sensor_id = HOTT_TELEMETRY_GAM_SENSOR_ID;
  hott_gam_msg->sensor_id = 0xd0;
  hott_gam_msg->stop_byte = 0x7d;
}

// Emission de la trame
void GMessage::send(int lenght){ 
  uint8_t sum = 0;
  delay(5);
  for(int i = 0; i < lenght-1; i++){
    sum = sum + _hott_serial_buffer[i];
    Serial.write (_hott_serial_buffer[i]);
    delayMicroseconds(HOTTV4_TX_DELAY);

    Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
  }  
  //Emision du checksum
  Serial.write (sum);
  delayMicroseconds(HOTTV4_TX_DELAY);
  Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
}

int GMessage::main(){ 
  // 
  static byte page_settings = 1;//numéro de la page de settings à afficher 
    
  if (Serial.available() >= 2) {
    uint8_t octet1 = Serial.read();
    switch (octet1) {
    case HOTT_BINARY_MODE_REQUEST_ID:
      { 
        uint8_t  octet2 = Serial.read();        
        // Demande RX Module =	$80 $XX
        switch (octet2) {
        case HOTT_TELEMETRY_GAM_SENSOR_ID: //0x8D    Traitement trame telemesure General Air Module
        {
          // init structure
          init_gam_msg();        
            
          // ******************************************************
          // LEDPIN_ON	      // LED allumée pendant traitement COM Telemesure
		      // ******************************************************
		      // ** Affichage des tensions en numérique ou jauge
		  	
          setVoltage1 (lipo1);    // Affichage sur Batt 1
		      setVoltage2 (lipo2);    // Affichage sur Batt 2
          setMainVoltage (lipo3); // Affichage sur Tension principale

          if (mode_gst_carburant==0) { // 0 = Reservoir / 1 = Quantite          
		        // Affichage jauge carburant entre 0% et 100% (Entier sur 8 bits unsigned (type uint8_t))   
            // Contenu du reservoir       
		        Jauge =  byte(float(volume_carburant) / float(capa_reservoir) * 100.0);			    
            setFuelPercent (Jauge);      
            // Affichage du volume carburant
		        setFuelMilliliters(volume_carburant);     
          }
          else      // 1 = Quantite  
          {
            // Affichage jauge carburant entre 0% et 100% (Entier sur 8 bits unsigned (type uint8_t))   
            // Consommation 
            Jauge =  100 - (byte(conso_carburant / float(qt_carburant) * 100.0));
            setFuelPercent (Jauge);  
            setFuelMilliliters(word(conso_carburant));        
          }
		  
		      // Gestion Alarme : envoie du code son ou message vocal correspondant          
          alarme (bip_alarme); 
          bip_alarme = 0; //  remise a zero de l'alarme apres chaque emission

          // altitude 0 et montée 0
          setAltitudeInMeters (0);
          setClimbrate (0);
          setClimbrate_M3 (0);  

          // Temperatures moteur capteurs CTN
          setTemp(0,1);
          setTemp(0,2);            
                          
          send(sizeof(struct HOTT_GAM_MSG));
		      LEDPIN_OFF 
          break;
       } //end case GAM
      } //end case octet 2
      break;
    } // End case Binary Mode

    case HOTT_TEXT_MODE_REQUEST_ID:
      {      
        uint8_t  octet3 = Serial.read();
        byte id_sensor = (octet3 >> 4);
        byte id_key = octet3 & 0x0f;
        static byte ligne_select = 2 ; // Ligne 1 selectionnée par défaut
        static int8_t ligne_edit = -1 ;
        hott_txt_msg->start_byte = 0x7b;
        hott_txt_msg->esc = 0;
        hott_txt_msg->warning_beeps = 0;
        
        memset((char *)&hott_txt_msg->text, 0x20, HOTT_TEXTMODE_MSG_TEXT_LEN);
        hott_txt_msg->stop_byte = 0x7d;
        
          if (id_sensor == (HOTT_TELEMETRY_GAM_SENSOR_ID & 0x0f)) 
          {
            switch (page_settings) { // Pages de parametres JetHott
              
              case 1:   //PAGE 1 
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(5,ligne_select + 1); // Ligne 5 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(2,ligne_select - 1); // Ligne 2 mini
                          break; 
                          
                        case HOTT_KEY_SET :                      // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :        // Sortie
                          hott_txt_msg->esc = 0x01;
                          ligne_edit = -1 ;
                          ligne_select = 1;
                          break;  
                                           
                        case HOTT_KEY_RIGHT :      // passage en page 2              
                          page_settings = 2; 
                          ligne_edit = -1 ;     
                          ligne_select = 2;                      
                          break;    
                        
                        default : break;
                      }
                    }   
                      
                    //*************************************************
                    //LIGNE 2 SELECTIONNEE = Seuil alarme (tension mini)
                    else if (ligne_select == 2)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          seuil_tension_alarme = seuil_tension_alarme +5;  
                          if (seuil_tension_alarme >800)			// 800 = 8.00 V
                            seuil_tension_alarme = 800;
                          break;
                          
                        case HOTT_KEY_DOWN :
                          seuil_tension_alarme = seuil_tension_alarme -5;  
                          if (seuil_tension_alarme <600)             // 600 = 6.00 V
							              seuil_tension_alarme = 600;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_seuil_tension_alarme,seuil_tension_alarme); // Ecriture en EEPROM
                          break;
                          
                        default : break;
                      }
                    }          
                                                                          
                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Repetition alarme
                    else if (ligne_select == 3)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          temps_repetition = temps_repetition + 5;
                          if (temps_repetition > 30)          // Temps en secondes
                            temps_repetition = 30;            // 30s maxi	     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          temps_repetition = temps_repetition - 5;
                          if (temps_repetition < 10) 
                            temps_repetition = 10; 			// 10s mini	 
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_repetition,temps_repetition); // Ecriture en memoire
                          break;                     
                          
                        default : break;
                      }
                    }                    

                    //*************************************************
                    //LIGNE 4 SELECTIONNEE = Alarme LiPo 2 ON / OFF
                    else if (ligne_select == 4)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (alarme_lipo_2 == 0)
                             alarme_lipo_2 = 1;                  
                          else
                             alarme_lipo_2 = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (alarme_lipo_2 == 0)
                             alarme_lipo_2 = 1;                  
                          else
                             alarme_lipo_2 = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarm_bat2,alarme_lipo_2); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }
					
                    //*************************************************
                    //LIGNE 5 SELECTIONNEE = Alarme LiPo 3 ON / OFF
                    else if (ligne_select == 5)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (alarme_lipo_3 == 0)
                             alarme_lipo_3 = 1;                  
                          else
                             alarme_lipo_3 = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (alarme_lipo_3 == 0)
                             alarme_lipo_3 = 1;                  
                          else
                             alarme_lipo_3 = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarm_bat3,alarme_lipo_3); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }
                    //******************************************************************************************
                    // Affichage de la page 1
                    //ligne 0:
                    snprintf((char *)&hott_txt_msg->text[0],21," * JetHott   V%s *",VERSION);
                    //ligne 1:
					          snprintf((char *)&hott_txt_msg->text[1],21,"  Gestion batteries ");
					          //ligne 2:
                    snprintf((char *)&hott_txt_msg->text[2],21," Seuil alarme  %i.%iV",int(seuil_tension_alarme/100),int(seuil_tension_alarme%100)) ;   //float(seuil_tension_alarme)/100.0);
                    //ligne 3:
                    snprintf((char *)&hott_txt_msg->text[3],21," Repet. alarme   %2is",temps_repetition);
                    //ligne 4:
                    if (alarme_lipo_2 == 1)
                        snprintf((char *)&hott_txt_msg->text[4],21," Alarme bat 2     ON");
                    else 
                        snprintf((char *)&hott_txt_msg->text[4],21," Alarme bat 2    OFF");
                    //ligne 5:
                    if (alarme_lipo_3 == 1)
                        snprintf((char *)&hott_txt_msg->text[5],21," Alarme bat 3     ON");
                    else 
                        snprintf((char *)&hott_txt_msg->text[5],21," Alarme bat 3    OFF");
                    //ligne 6:
                    snprintf((char *)&hott_txt_msg->text[6],21," ------------------- ");
                    //ligne 7:
                    snprintf((char *)&hott_txt_msg->text[7],21,"                2019"); 
                    
                    hott_txt_msg->text[ligne_select][0] = '>';
                    _hott_invert_ligne(ligne_edit);
                    break;                    
              }//END PAGE 1
			  
			  
			  //-------------------------------------------------------------------------------------------------    
              case 2:             //PAGE 2 
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(6,ligne_select + 1); // // Ligne 6 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(2,ligne_select - 1); // // Ligne 2 mini
                          break; 
                          
                        case HOTT_KEY_SET :   // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :    // passage en page 1  
                          page_settings = 1;
                          ligne_edit = -1;
                          ligne_select = 2;                           
                          break;
                          
                         case HOTT_KEY_RIGHT :  // derniere page                 
                          break;  
                                                   
                        default : break;
                      }
                    }   

                   //*************************************************
                    //LIGNE 2 SELECTIONNEE = Capacite du reservoir de carburant ou quantite carburant a consommer
                    else if (ligne_select == 2)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (mode_gst_carburant == 0) {   // Mode reservoir
                            capa_reservoir = capa_reservoir + 100;     // ml
                            if (capa_reservoir > 65000)
                              capa_reservoir = 65000;                  // Max : 6.5 litres
                          }
                          else {       // Mode quantite
                            qt_carburant = qt_carburant + 100;     // ml
                            if (qt_carburant > 65000)
                              qt_carburant = 65000;                  // Max : 6.5 litres
                          }
                              
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (mode_gst_carburant == 0) {      // Mode reservoir
                            capa_reservoir = capa_reservoir - 100;     // ml
                            if (capa_reservoir < 200)
                              capa_reservoir = 200;                    // min 0.2 l
                          }
                          else {             // Mode quantite
                            qt_carburant = qt_carburant - 100;     // ml
                            if (qt_carburant  < 200)
                              qt_carburant = 200;                    // min 0.2 l    
                          }
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          if (mode_gst_carburant == 0)    {        // Mode reservoir
                            write_eprom(adr_eprom_capa_reservoir,capa_reservoir); // Ecriture EEPROM
                            volume_carburant = capa_reservoir; 
                          }  
                          else               // Mode quantite
                            write_eprom(adr_eprom_qt_carburant,qt_carburant);         
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }

                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Seuil pour alarme carburant en mode Reservoir seulement
                    else if ((ligne_select == 3) && (mode_gst_carburant == 0))
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          seuil_reservoir = seuil_reservoir + 50;     // ml
                          if (seuil_reservoir > 20000)
                            seuil_reservoir = 20000;                  // Max : 2.0 litres
                          break;
                          
                        case HOTT_KEY_DOWN :
                          seuil_reservoir = seuil_reservoir - 50;     // ml
                          if (seuil_reservoir < 50)
                            seuil_reservoir = 50;                    // min 0.05 l
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_seuil_reservoir,seuil_reservoir); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }                   
                        
                    //*************************************************
                    //LIGNE 4 SELECTIONNEE = Alarme carburant ON/OFF
                    else if (ligne_select == 4)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (alarme_carburant == 0)
                             alarme_carburant = 1;                  
                          else
                             alarme_carburant = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (alarme_carburant == 0)
                             alarme_carburant = 1;                  
                          else
                             alarme_carburant = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarm_carburant,alarme_carburant); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 
                    
                    //*************************************************
                    //LIGNE 5 SELECTIONNEE = Debitmetre / Nb impulsions par litre
                    else if (ligne_select == 5)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          impulsions_litre = impulsions_litre + 50;     // 
                          if (impulsions_litre > 20000)
                            impulsions_litre = 20000;                  // Max 
                          break;
                          
                        case HOTT_KEY_DOWN :
                          impulsions_litre = impulsions_litre - 50;     // 
                          if (impulsions_litre < 5000)
                            impulsions_litre = 5000;                    // min 
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_impulsions_litre,impulsions_litre); // Ecriture EEPROM
                          imp_ml = float(impulsions_litre) / 1000.0;     // mise à jour impulsions / ml
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 					
	
                    //*************************************************
                    //LIGNE 6 SELECTIONNEE = Mode Reservoir ou Quantite
                    else if (ligne_select == 6)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (mode_gst_carburant == 0)
                             mode_gst_carburant = 1;                  
                          else
                             mode_gst_carburant = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (mode_gst_carburant == 0)
                             mode_gst_carburant = 1;                  
                          else
                             mode_gst_carburant = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_mode_carburant,mode_gst_carburant); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 	
					          
					//******************************************************************************************
                    // Affichage de la page 2
                    //ligne 0:
                    snprintf((char *)&hott_txt_msg->text[0],21," * JetHott   V%s *",VERSION);
					          //ligne 1:
                    snprintf((char *)&hott_txt_msg->text[1],21,"  Gestion carburant ");
                    //ligne 2:
					          if (mode_gst_carburant==0)  // 0 = Reservoir / 1 = Quantite
                      snprintf((char *)&hott_txt_msg->text[2],21," Reservoir  %5i ml",capa_reservoir);
					          else
				              snprintf((char *)&hott_txt_msg->text[2],21," Quantite   %5i ml",qt_carburant);
                    //ligne 3:
					          if (mode_gst_carburant==0)  // 0 = Reservoir / 1 = Quantite
                      snprintf((char *)&hott_txt_msg->text[3],21," Seuil carb %5i ml",seuil_reservoir);
					          else
                      snprintf((char *)&hott_txt_msg->text[3],21,"                    ");						  
                    //ligne 4:
                    if (alarme_carburant == 1)
                        snprintf((char *)&hott_txt_msg->text[4],21," Alarme carb.     ON");
                    else 
                        snprintf((char *)&hott_txt_msg->text[4],21," Alarme carb.    OFF");
                    //ligne 5:
                    snprintf((char *)&hott_txt_msg->text[5],21," Impulsions/l  %5i",impulsions_litre); 
                    //ligne 6:
					          if (mode_gst_carburant==0)  // 0 = Reservoir / 1 = Quantite
                      snprintf((char *)&hott_txt_msg->text[6],21," Mode Reservoir     ");
					          else
                      snprintf((char *)&hott_txt_msg->text[6],21," Mode Quantite      "); 						
                    //ligne 7:
                    snprintf((char *)&hott_txt_msg->text[7],21,"                    ");

                    hott_txt_msg->text[ligne_select][0] = '>';
                    _hott_invert_ligne(ligne_edit);
                    break;   
                   }//END PAGE 2 
				   
                  default: //PAGE 
                  {
                    break;
                  }            
                 
            }//END SETTINGS 
             // Emission en mode texte
			       _hott_send_text_msg();  // Trame GAM
				   
          } // END IF :  id_sensor = HOTT_TELEMETRY_GAM_SENSOR_ID 
          
        break;
      }   // Case HOTT_TEXT_MODE_REQUEST_ID
     }  // switch (octet1)	
	return (1);
  }
  else
    return (0); 
}

//////////////////////////////////////////////////
// Tension principale (resolution 0.1V)
void GMessage::setMainVoltage(float tension){
  hott_gam_msg->main_voltage = (uint16_t) (tension * 10);
}

// Batterie 1 (resolution 0.1V)
void GMessage::setVoltage1(float volt){
  hott_gam_msg->Battery1 =  (uint16_t) (volt * 10); 
}

// Batterie 2 (resolution 0.1V)
void GMessage::setVoltage2(float volt){
  hott_gam_msg->Battery2 = (uint16_t) (volt * 10); 
}

// Element Lipo n (resolution 0.02V)
void GMessage::setLipo(float volt, int lipo){

  if (lipo >= 1 and lipo <= 6)
  {
    lipo--;
    hott_gam_msg->cell[lipo] =  (uint16_t) 100 * volt / 2 ; 
    
    if (volt/2 <= hott_gam_msg->min_cell_volt || hott_gam_msg->min_cell_volt ==0)
    {
      hott_gam_msg->min_cell_volt = (uint16_t) 100 * volt/2 ;
      hott_gam_msg->min_cell_volt_num = lipo;
    }
  }
}

// Altitude relative en metres : -500 / +9999 [m]
void GMessage::setAltitudeInMeters(uint16_t alti){
  hott_gam_msg->altitude = alti + 500;
}

// Temperature : -20 a +235° C 
// 
void GMessage::setTemp(int temp, int capteur){
  if(temp < -20)
    temp = -20;
  else if(temp > 234)
    temp = 235;
  if (capteur < 1)
    capteur = 1;
  if (capteur > 2)
    capteur = 2; 
  if (capteur == 1)
    hott_gam_msg->temperature1 = temp + 20;
  else if (capteur == 2)
    hott_gam_msg->temperature2 = temp + 20;
}

// Niveau reservoir : 0%, 25%, 50%, 75%, 100%  
// 
void GMessage::setFuelPercent(uint8_t pourcent){
  if(pourcent > 100)
    pourcent = 100;
  else if(pourcent < 0)
    pourcent = 0;
  hott_gam_msg->fuel_procent = pourcent ;
}

// niveau reservoir en ml : 0 a 65535 ml
// 
void GMessage::setFuelMilliliters(uint16_t ml){
  hott_gam_msg->fuel_ml = ml ;
}

// Rotation : 0 a 655350 tr/min
// 

void GMessage::setRPM(uint16_t rpm){
  hott_gam_msg->rpm = rpm ;
} 

// Set climbrates
void GMessage::setClimbrate(uint16_t climb_L){
  hott_gam_msg->climbrate_L =  climb_L+30000;
}

void GMessage::setClimbrate_M3(int climb3s){
  hott_gam_msg->climbrate3s =  climb3s+120;
}

// Set speed
void GMessage::setSpeed(float speed){
  hott_gam_msg->climbrate3s =  (int) speed;
}


//
// Alarme sonore sur radio
//
void GMessage::alarme (uint8_t son){
  hott_gam_msg->warning_beeps = son;
}

void GMessage::_hott_send_text_msg() {
  for(byte *_hott_msg_ptr = hott_txt_msg->text[0]; _hott_msg_ptr < &hott_txt_msg->stop_byte ; _hott_msg_ptr++){
    if (*_hott_msg_ptr == 0)
      *_hott_msg_ptr = 0x20;
  }  
  //_hott_send_msg(_hott_serial_buffer, sizeof(struct HOTT_TEXTMODE_MSG));
  send(sizeof(struct HOTT_TEXTMODE_MSG));
}

char * GMessage::_hott_invert_all_chars(char *str) {
  return _hott_invert_chars(str, 0);
}

char * GMessage::_hott_invert_chars(char *str, int cnt) {
  if(str == 0) return str;
  int len = strlen(str);
  if((len < cnt)  && cnt > 0) len = cnt;
  for(int i=0; i< len; i++) {
    str[i] = (byte)(0x80 + (byte)str[i]);
  }
  return str;
}

void GMessage::_hott_invert_ligne(int ligne) {
  if (ligne>= 0 && ligne<= 7)
    for(int i=0; i< 21; i++) {
      if (hott_txt_msg->text[ligne][i] == 0)   //inversion du caratère null (fin de string)
        hott_txt_msg->text[ligne][i] = (byte)(0x80 + 0x20);
      else
        hott_txt_msg->text[ligne][i] = (byte)(0x80 + (byte)hott_txt_msg->text[ligne][i]);
    }
}
